class AST::FunctionCall
  attr_reader :function, :arguments

  def initialize(function, arguments)
    @function = function
    @arguments = arguments
  end

  def ==(other)
    other.is_a?(AST::FunctionCall) && other.function == @function &&
      other.arguments == @arguments
  end

  def execute(env)
    env = Environment.new(env)

    function = if @function.is_a?(AST::Identifier)
      env.get(@function.name)
    elsif @function.is_a?(AST::FunctionDefinition)
      @function
    end

    if function.is_a?(AST::FunctionDefinition)
      env = function.env unless function.env.nil?
      if @arguments.size != function.parameters.size
        raise "Arity mismatch in call to function #{function.name}"
      end

      function.parameters.each_with_index do |param, i|
        env.set(param.name, @arguments[i].execute(env))
      end

      value = nil

      function.body.statements.each do |statement|
        value = statement.execute(env)
      end

      value
    elsif function.is_a?(Proc)
      arguments = @arguments.map { |arg| arg.execute(env) }
      function.call(*arguments)
    end
  end
end
